@@ -29,7 +29,7 @@ snp_ldpred2_inf <- function(corr, df_beta, h2) {
   assert_pos(h2, strict = TRUE)
 
   N <- df_beta$n_eff
-  scale <- df_beta$beta_se * sqrt(N)
+  scale <- sqrt(N * df_beta$beta_se^2 + df_beta$beta^2)
   beta_hat <- df_beta$beta / scale
 
   beta_inf <- bigsparser::sp_solve_sym(
@@ -48,9 +48,15 @@ snp_ldpred2_inf <- function(corr, df_beta, h2) {
 #' @param burn_in Number of burn-in iterations.
 #' @param num_iter Number of iterations after burn-in.
 #' @inheritParams bigsnpr-package
+#' @param return_sampling_betas Whether to return all sampling betas (after
+#'   burn-in)? This is useful for assessing the uncertainty of the PRS at the
+#'   individual level (see \doi{10.1101/2020.11.30.403188}).
+#'   Default is `FALSE` (only returns the averaged final vectors of betas).
+#'   If `TRUE`, only one set of parameters is allowed.
 #'
 #' @return `snp_ldpred2_grid`: A matrix of effect sizes, one vector (column)
-#'   for each row of `grid_param`.
+#'   for each row of `grid_param`. If using `return_sampling_betas`, each
+#'   column corresponds to one iteration instead (after burn-in).
 #'
 #' @export
 #'
@@ -59,7 +65,8 @@ snp_ldpred2_inf <- function(corr, df_beta, h2) {
 snp_ldpred2_grid <- function(corr, df_beta, grid_param,
                              burn_in = 50,
                              num_iter = 100,
-                             ncores = 1) {
+                             ncores = 1,
+                             return_sampling_betas = FALSE) {
 
   assert_df_with_names(df_beta, c("beta", "beta_se", "n_eff"))
   assert_df_with_names(grid_param, c("p", "h2", "sparse"))
@@ -69,27 +76,46 @@ snp_ldpred2_grid <- function(corr, df_beta, grid_param,
   assert_cores(ncores)
 
   N <- df_beta$n_eff
-  scale <- df_beta$beta_se * sqrt(N)
+  scale <- sqrt(N * df_beta$beta_se^2 + df_beta$beta^2)
   beta_hat <- df_beta$beta / scale
 
-  # compute one infinitesimal model, just for initialization
-  med_h2 <- stats::median(grid_param$h2)
-  beta_inf <- bigsparser::sp_solve_sym(
-    corr, beta_hat, add_to_diag = ncol(corr) / (med_h2 * N))
+  if (!return_sampling_betas) {
 
-  beta_gibbs <- ldpred2_gibbs(
-    corr      = corr,
-    beta_hat  = beta_hat,
-    beta_init = beta_inf,
-    order     = order(beta_inf^2, decreasing = TRUE) - 1L,
-    n_vec     = N,
-    h2        = grid_param$h2,
-    p         = grid_param$p,
-    sparse    = grid_param$sparse,
-    burn_in   = burn_in,
-    num_iter  = num_iter,
-    ncores    = ncores
-  )
+    # LDpred2-grid models
+    beta_gibbs <- ldpred2_gibbs(
+      corr      = corr,
+      beta_hat  = beta_hat,
+      beta_init = rep(0, length(beta_hat)),
+      order     = seq_along(beta_hat) - 1L,
+      n_vec     = N,
+      h2        = grid_param$h2,
+      p         = grid_param$p,
+      sparse    = grid_param$sparse,
+      burn_in   = burn_in,
+      num_iter  = num_iter,
+      ncores    = ncores
+    )
+
+  } else {
+
+    if (nrow(grid_param) != 1)
+      stop2("Only one set of parameters is allowed when using 'return_sampling_betas'.")
+
+    # All sampling betas for one LDpred2 model (useful for uncertainty assessment)
+    beta_gibbs <- ldpred2_gibbs_one_sampling(
+      corr      = corr,
+      beta_hat  = beta_hat,
+      beta_init = rep(0, length(beta_hat)),
+      order     = seq_along(beta_hat) - 1L,
+      n_vec     = N,
+      h2        = grid_param$h2,
+      p         = grid_param$p,
+      sparse    = grid_param$sparse,
+      burn_in   = burn_in,
+      num_iter  = num_iter
+    )
+
+  }
 
   sweep(beta_gibbs, 1, scale, '*')
 }
@@ -105,12 +131,15 @@ snp_ldpred2_grid <- function(corr, df_beta, grid_param,
 #'
 #' @return `snp_ldpred2_auto`: A list (over `vec_p_init`) of lists with
 #'   - `$beta_est`: vector of effect sizes
+#'   - `$beta_est_sparse` (only when `sparse = TRUE`): sparse vector of effect sizes
+#'   - `$postp_est`: vector of posterior probabilities of being causal
 #'   - `$p_est`: estimate of p, the proportion of causal variants
 #'   - `$h2_est`: estimate of the (SNP) heritability (also see [coef_to_liab])
 #'   - `$path_p_est`: full path of p estimates (including burn-in);
 #'     useful to check convergence of the iterative algorithm
 #'   - `$path_h2_est`: full path of h2 estimates (including burn-in);
 #'     useful to check convergence of the iterative algorithm
+#'   - `$h2_init` and `$p_init`
 #'
 #' @import foreach
 #'
@@ -132,13 +161,9 @@ snp_ldpred2_auto <- function(corr, df_beta, h2_init,
   assert_pos(h2_init, strict = TRUE)
 
   N <- df_beta$n_eff
-  scale <- df_beta$beta_se * sqrt(N)
+  scale <- sqrt(N * df_beta$beta_se^2 + df_beta$beta^2)
   beta_hat <- df_beta$beta / scale
 
-  # compute one infinitesimal model, just for initialization
-  beta_inf <- bigsparser::sp_solve_sym(
-    corr, beta_hat, add_to_diag = ncol(corr) / (h2_init * N))
-
   bigparallelr::register_parallel(ncores)
 
   foreach(p_init = vec_p_init,
@@ -147,8 +172,8 @@ snp_ldpred2_auto <- function(corr, df_beta, h2_init,
     ldpred_auto <- ldpred2_gibbs_auto(
       corr      = corr,
       beta_hat  = beta_hat,
-      beta_init = beta_inf,
-      order     = order(beta_inf^2, decreasing = TRUE) - 1L,
+      beta_init = rep(0, length(beta_hat)),
+      order     = seq_along(beta_hat) - 1L,
       n_vec     = N,
       p_init    = p_init,
       h2_init   = h2_init,
@@ -156,14 +181,17 @@ snp_ldpred2_auto <- function(corr, df_beta, h2_init,
       num_iter  = num_iter,
       verbose   = verbose
     )
-    ldpred_auto$beta_est <- drop(ldpred_auto$beta_est) * scale
+    ldpred_auto$beta_est  <- drop(ldpred_auto$beta_est) * scale
+    ldpred_auto$postp_est <- drop(ldpred_auto$postp_est)
+    ldpred_auto$h2_init <- h2_init
+    ldpred_auto$p_init  <- p_init
 
     if (sparse) {
       beta_gibbs <- ldpred2_gibbs(
         corr      = corr,
         beta_hat  = beta_hat,
-        beta_init = beta_inf,
-        order     = order(beta_inf^2, decreasing = TRUE) - 1L,
+        beta_init = rep(0, length(beta_hat)),
+        order     = seq_along(beta_hat) - 1L,
         n_vec     = N,
         h2        = ldpred_auto$h2_est,
         p         = ldpred_auto$p_est,
